<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardocarvalho
 * Date: 15/05/14
 * Time: 18:06
 * To change this template use File | Settings | File Templates.
 */

namespace Importacao\Logic\Bases\ModeloImportacao;


class MigraTabelasDefinitivas {

    const GET = "get";
    const SET = "set";

    private $estruturaJson;
    private $agrupadores;
    private $nrorg;
    private $_notGetters = Array('getNrprojimport','getObject','getObjectsByWhere','getObjectByWhere','getRepository');
    private $_notSetters = Array('setNrprojimport');

    /**
     * @param $estruturaJson
     */
    public function __construct($estruturaJson)
    {
        $this->estruturaJson = $estruturaJson;
        $this->nrorg         = \Importacao\Logic\Integracao\ProjetoImportacao::getNrorg();
    }

    /**
     * @param $agrupador
     */
    public function addAgrupador($agrupador)
    {
        $this->agrupador = $agrupador;
    }

    public function build()
    {
        $this->percorreAgrupamentos();
    }

    /**
     * Percorre os agrupamento do Estrutura.json, compara com o agrupamento atual e pecorre suas tabelas
     */
    private function percorreAgrupamentos()
    {
        foreach ($this->estruturaJson as $nomeAgrupamento => $tabelas) {
            if ($this->agrupador === $nomeAgrupamento) {
                $this->percorreTabelas($tabelas);
            }
        }
    }

    /**
     * Percorre as tabelas de migração, retornando seus dados e persistindo respetivas tabelas definitivas
     *
     * @param $tabelas
     */
    private function percorreTabelas($tabelas)
    {
        foreach ($tabelas as $nomeTabela => $dadosTabela) {
            \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

            // define nome das tabelas de migração e definitivas
            $tabelaMigracao   = \Importacao\Logic\Util\Util::normalizeNameClass($nomeTabela);
            $tabelaDefinitiva = $this->nameClassDefinitiva($nomeTabela);

            // retorna os dados da tabela pelo número da organização
            $classNameMigracao   = "\\Folha\\Entities\\".$tabelaMigracao;
            $dadosTabelaMigracao = $classNameMigracao::getObjectsByWhere('NRORG = '.$this->nrorg);

            // retorna todos os metodos getters da classe instanciada
            $classMethods = get_class_methods($classNameMigracao);
            $getters      = preg_grep("/^get\w+/", $classMethods);

            // remove do arrau $getters as posições definidas no array $this->_notGetters
            $getters = $this->removeInArray($getters, $this->_notGetters);

            // define classe da tabela definitiva
            $classNameDefinitiva = $this->returnObjectClass($tabelaDefinitiva);

            // percorre os dados da tabela de migração
            $this->percorreDados($dadosTabelaMigracao, $classNameDefinitiva, $getters);

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            \Setforms2\Shared\Registry::getEntityManager()->commit();
        }
    }

    /**
     * Percorre os dados da tabela de migração e persit os valores na tabela definitiva
     *
     * @param $dadosTabelaMigracao
     * @param $classNameDefinitiva
     * @param $getters
     * @internal param $objectClassDefinitiva
     * @internal param $classNameDefinitiva
     */
    private function percorreDados($dadosTabelaMigracao, $classNameDefinitiva, $getters)
    {
        // percorre dados da tabela de migração
        foreach ($dadosTabelaMigracao as $value) {
            $table = new $classNameDefinitiva();

            // percorre array de métodos getters da entidade
            foreach ($getters as $getField => $valor) {
                $setField = str_replace(self::GET, self::SET, $getField);

                // seta os campos na entidade definitiva passando seu determinado valor da tabela de migração
                $table->$setField(trim($value->$getField()));
            }

            $table->persist();
        }
    }

    /**
     * Trata o nome da tabela, retornando com o padrão do Doctrine
     *
     * @param $tabela
     * @return string
     */
    private static function nameClassDefinitiva($tabela) {
        $tabela = explode("_", $tabela);

        $nomeDaClasse  = ucfirst(strtolower($tabela[0]));
        $nomeDaClasse .= ucfirst(strtolower($tabela[1]));

        return $nomeDaClasse;
    }

    /**
     * Remove posições do array, a partir dos valores do segundo array passado como parâmetro
     *
     * @param $arrayInteiro
     * @param $arrayUnset
     * @return array
     */
    private function removeInArray($arrayInteiro, $arrayUnset) {

        $arrayInteiro = array_fill_keys($arrayInteiro, '0');

        foreach ($arrayUnset as $value) {
            unset($arrayInteiro[$value]);
        }

        return $arrayInteiro;
    }

    /**
     * @param $tabelaDefinitiva
     * @return string
     */
    private function returnObjectClass($tabelaDefinitiva)
    {
        $folhaEntities = "\\Folha\\Entities\\" . $tabelaDefinitiva;
        $geralEntities = "\\Geral\\Entities\\" . $tabelaDefinitiva;

        if (class_exists($folhaEntities)) {
            $classNameDefinitiva = $folhaEntities;
        } elseif (class_exists($geralEntities)) {
            $classNameDefinitiva = $geralEntities;
        }

        return $classNameDefinitiva;
    }
}