<?php
/**
 * Created by PhpStorm.
 * User: rodrigoarruda
 * Date: 16/05/14
 * Time: 09:50
 */

namespace Planejamento\Logic;


use \Geral\Entities\Lista;
use \Geral\Util\Ambiente;
use Planejamento\Entities\Confcard;
use \Planejamento\Entities\Custpadr;
use \Geral\Util\DateUtil;
use Setforms2\Shared\Registry;

class Servico
{

    CONST TAB = 'tab_ger04102_000_001';
    CONST WINDOW = 'win_ger04102_000';
    const TODOS = 'A';
    const SIM = 'S';

    /**
     * Retorna o valor do multisearch da filial concatenado
     *
     * @param $servico
     *
     * @return array
     */
    public static function retornaServicoMultiSearch($servico)
    {
        $cdservfili = '';
        $tcdservfili = '';

        if ($servico == 'A') {
            $tcdservfili = 'A';
            return array($cdservfili, $tcdservfili);
        } elseif ($servico == 'N') {
            return array($cdservfili, $tcdservfili);
        } elseif (is_array($servico)) {
            $concatenado = '_';
            foreach ($servico as $f) {
                $concatenado .= $f['cdservfili'] . '_';
            }
            $cdservfili = $concatenado;
            return array($cdservfili, $tcdservfili);
        }
    }

    /**
     * Retorna o valor do multisearch da filial concatenado
     *
     * @param $filial
     *
     * @return array
     */
    public static function retornaFilialMultiSearch($filial)
    {
        //$filial = tk_get_field_value('CDFILIAL');
        $cdfilial = '';
        $tcdfilial = '';

        if ($filial == 'A') {
            $tcdfilial = 'A';
            return array($cdfilial, $tcdfilial);
        } elseif ($filial == 'N') {
            return array($cdfilial, $tcdfilial);
        } elseif (is_array($filial)) {
            $concatenado = '_';
            foreach ($filial as $f) {
                $concatenado .= $f['cdfilial'] . '_';
            }
            $cdfilial = $concatenado;
            return array($cdfilial, $tcdfilial);
        }
    }

    public static function geraLista()
    {
        $nrlista = Lista::novoCodigo();
        return $nrlista;
    }

    /**
     * Realiza a inserção das unidades concatenadas com o serviço na lista
     *
     * @param $nrlista
     * @param $filial
     * @param $servico
     */
    public static function trataFilial($nrlista, $filial, $servico)
    {
        $nrorg = Ambiente::getNrorg();

        if ($filial == 'N') {
            tk_abort(tk_get_message_by_name('msg_ger04102_000_Filial_N'));
        } else {
            if ($filial == 'A') {
                if ($servico === 'A') {
                    /** @var  $servicos \Geral\Entities\Servico[] */
                    $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                        'NRORG = ?', array($nrorg)
                    );
                    foreach ($servicos as $servico) {
                        Lista::insere(
                            $nrlista,
                            $servico->getCdfilial() . $servico->getCdservfili(),
                            $nrorg
                        );
                    }
                } else {
                    if ($servico === 'N') {
                        tk_abort(
                            tk_get_message_by_name('msg_ger04102_000_Servico_N')
                        );
                    } else {
                        foreach ($servico as $servfili) {
                            Lista::insere(
                                $nrlista,
                                $servfili['cdfilial'] . $servfili['cdservfili'],
                                $nrorg
                            );
                        }
                    }
                }
            } else {
                foreach ($filial as $fili) {
                    if ($servico === 'A') {
                        /** @var  $servicos \Geral\Entities\Servico[] */
                        $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                            'CDFILIAL = ? AND NRORG = ? AND HRINICATEN IS NOT NULL',
                            array($fili['cdfilial'], $nrorg)
                        );

                        foreach ($servicos as $serv) {
                            Lista::insere(
                                $nrlista,
                                $serv->getCdfilial() . $serv->getCdservfili(),
                                $nrorg
                            );
                        }
                    } else {
                        if ($servico === 'N') {
                            tk_abort(
                                tk_get_message_by_name(
                                    'msg_ger04102_000_Servico_N'
                                )
                            );
                        } else {
                            foreach ($servico as $servfili) {
                                Lista::insere(
                                    $nrlista, $servfili['cdfilial']
                                    . $servfili['cdservfili'],
                                    $nrorg
                                );
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Realiza validação das linhas que seram atualizadas ou inseridas ou ambos
     * os casos na Tela de Manutenção automática de custo Padrão
     *
     * @param $linhas
     * @param $sobrepor
     * @param $p_dtini
     * @param $p_dtfin
     * @param $porcentagem
     * @param $linhasmarcadas
     * @param $idopcao
     * @param $iddestino
     *
     * @return null
     */
    public static function validaAtualizacao(
        $linhas, $sobrepor, $p_dtini, $p_dtfin, $porcentagem, $linhasmarcadas,
        $idopcao, $iddestino
    )
    {
        $atualizacao = false;
        foreach ($linhas as $registro) {
            if ($registro['VRCUSTPADR'] > 0) {
                $atualizacao = true;
                break;
            }
        }
        foreach ($linhasmarcadas as $registro) {
            if ($registro['VRCUSTPADR'] > 0) {
                $atualizacao = true;
                break;
            }
        }

        if ($idopcao == 'I') {
            if ($atualizacao == true) {
                if ($sobrepor == 'S') {
                    $message = tk_get_message_by_name(
                        'msg_ger04102_custosDoTipodeprato'
                    );
                    $yes = array(
                        'label' => tk_get_message_by_name('msg_sim'),
                        'method' => '\Planejamento\Logic\Servico::realizaManutencaoSobrepor',
                        'params_arr' => array(
                            $linhas, $porcentagem, $p_dtini, $p_dtfin,
                            $linhasmarcadas, $idopcao, $sobrepor
                        )
                    );
                    $no = array('label' => tk_get_message_by_name('msg_nao'),
                        'params_arr' => array());
                    tk_confirm($message, $yes, $no);
                } else {
                    if ($sobrepor == 'N') {
                        $message = tk_get_message_by_name(
                            'msg_ger04102_custos_cadastrados'
                        );
                        $yes = array(
                            'label' => tk_get_message_by_name('msg_sim'),
                            'method' => '\Planejamento\Logic\Servico::realizaManutencao',
                            'params_arr' => array(
                                $linhas, $p_dtini, $p_dtfin, $porcentagem,
                                $linhasmarcadas, $idopcao, $sobrepor
                            )
                        );
                        $no = array(
                            'label' => tk_get_message_by_name('msg_nao'),
                            'method' => '\Planejamento\Logic\Servico::somenteInsere',
                            'params_arr' => array(
                                $linhas, $p_dtini, $p_dtfin, $porcentagem,
                                $linhasmarcadas, $idopcao, $sobrepor
                            )
                        );
                        tk_confirm($message, $yes, $no);
                    }
                }
            } else {
                Servico::somenteInsere(
                    $linhas, $p_dtini, $p_dtfin, $porcentagem, $linhasmarcadas,
                    $idopcao, $sobrepor
                );
            }
        } else {
            if (empty($linhas) == true AND empty($porcentagem) == false) {
                if ($iddestino == 'P') {
                    Servico::atualizaNovoValorPercentual(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                    tk_reload_data_with_filter(
                        'tab_ger04102_000_001', self::WINDOW
                    );
                    tk_reload_data_with_filter(
                        'tab_pla01500_000_003', 'ger04100'
                    );
                    tk_message(
                        tk_get_message_by_name(
                            'msg_ger04102_vrcustpadr_finalizado'
                        ), 'success'
                    );
                    Servico::verificaValorPrato(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                }
                if ($iddestino == 'V') {
                    foreach ($linhasmarcadas as $registro) {
                        Servico::atualizaNovoValor(
                            $registro, $p_dtini, $p_dtfin, $porcentagem,
                            $idopcao, $sobrepor
                        );
                        tk_reload_data_with_filter(
                            'tab_ger04102_000_001', self::WINDOW
                        );
                        tk_reload_data_with_filter(
                            'tab_pla01500_000_003', 'ger04100'
                        );
                        tk_message(
                            tk_get_message_by_name(
                                'msg_ger04102_vrcustpadr_finalizado'
                            ), 'success'
                        );
                        Servico::verificaValorPrato(
                            $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                            $idopcao, $sobrepor
                        );
                    }
                }

            } else {
                if (empty($linhas) == false) {
                    foreach ($linhasmarcadas as $registro) {
                        Servico::atualizaNovoValor(
                            $registro, $p_dtini, $p_dtfin, $porcentagem,
                            $idopcao, $sobrepor
                        );
                        tk_reload_data_with_filter(
                            'tab_ger04102_000_001', self::WINDOW
                        );
                        tk_reload_data_with_filter(
                            'tab_pla01500_000_003', 'ger04100'
                        );
                        tk_message(
                            tk_get_message_by_name(
                                'msg_ger04102_vrcustpadr_finalizado'
                            ), 'success'
                        );
                        Servico::verificaValorPrato(
                            $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                            $idopcao, $sobrepor
                        );
                    }
                }
            }
        }

    }

    /**
     * Faz a inserção de um Novo Valor num serviço que não possui custo
     * cadastrado
     *
     * @param $linhas
     * @param $p_dtini
     * @param $p_dtfin
     * @param $porcentagem
     * @param $linhasmarcadas
     * @param $idopcao
     * @param $sobrepor
     */
    public static function somenteInsere(
        $linhas, $p_dtini, $p_dtfin, $porcentagem, $linhasmarcadas, $idopcao,
        $sobrepor
    )
    {
        if (empty($linhas)) {
            foreach ($linhasmarcadas as $registro) {
                if ($registro['VRCUSTPADR'] == null OR
                    $registro['VRCUSTPADR'] == ''
                ) {
                    Servico::insereNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                        $sobrepor
                    );
                }
            }
        }
        foreach ($linhas as $registro) {
            if ($registro['VRCUSTPADR'] == null OR
                $registro['VRCUSTPADR'] == ''
            ) {
                Servico::insereNovoValor(
                    $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                    $sobrepor
                );
                Servico::verificaValorPrato(
                    $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                    $sobrepor
                );
            }
        }
        tk_reload_data_with_filter('tab_ger04102_000_001', self::WINDOW);
        tk_message(
            tk_get_message_by_name(
                'msg_ger04102_vrcustpadr_finalizado'
            ), 'success'
        );
    }

    /**
     * O método faz a atualização e inserção simultâneas das linhas de acordo
     * com o campo do novo valor ou do campo de % sendo valor ou porcentagem. E
     * ao fim atualiza os dados na tela
     *
     * @param $linhas
     * @param $p_dtini
     * @param $p_dtfin
     * @param $porcentagem
     * @param $linhasmarcadas
     * @param $idopcao
     * @param $sobrepor
     */
    public static function realizaManutencao(
        $linhas, $p_dtini, $p_dtfin, $porcentagem, $linhasmarcadas, $idopcao,
        $sobrepor
    )
    {
        if (empty($linhas)) {
            foreach ($linhasmarcadas as $registro) {
                if ($registro['VRCUSTPADR'] == null OR
                    $registro['VRCUSTPADR'] == ''
                ) {
                    Servico::insereNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                } else {
                    Servico::atualizaNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                }
            }
        } else {
            foreach ($linhas as $registro) {
                if ($registro['VRCUSTPADR'] == '') {
                    Servico::insereNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                        $sobrepor
                    );
                } else {
                    Servico::atualizaNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                        $sobrepor
                    );
                }
            }
        }
        tk_reload_data_with_filter('tab_ger04102_000_001', self::WINDOW);
        tk_reload_data_with_filter('tab_pla01500_000_003', 'ger04100');
        /* ID msg 1001331*/
        tk_message(
            tk_get_message_by_name('msg_ger04102_vrcustpadr_finalizado'),
            'success'
        );
    }

    /**
     * O método faz a atualização e inserção simultâneas das linhas de acordo
     * com o campo do novo valor ou do campo de % sendo valor ou porcentagem. E
     * ao fim atualiza os dados na tela. Este método trata as inserções e
     * atualizações com o parametro de sobrepor marcado.
     *
     * @param $linhas
     * @param $porcentagem
     * @param $p_dtini
     * @param $p_dtfin
     * @param $linhasmarcadas
     * @param $idopcao
     * @param $sobrepor
     */
    public static function realizaManutencaoSobrepor(
        $linhas, $porcentagem, $p_dtini, $p_dtfin, $linhasmarcadas, $idopcao,
        $sobrepor
    )
    {
        if (empty($linhas)) {
            foreach ($linhasmarcadas as $registro) {
                if ($registro['VRCUSTPADR'] == null OR
                    $registro['VRCUSTPADR'] == ''
                ) {
                    Servico::insereNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                } else {
                    Servico::atualizaNovoValor(
                        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao,
                        $sobrepor
                    );
                    Servico::verificaValorPrato(
                        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin,
                        $idopcao, $sobrepor
                    );
                }
            }
        }
        foreach ($linhas as $registro) {
            if ($registro['VRCUSTPADR'] == '') {
                Servico::insereNovoValorSobrepor($registro, $porcentagem);
                Servico::verificaValorPrato(
                    $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                    $sobrepor
                );
            } else {
                Servico::atualizaNovoValorSobrepor($registro, $porcentagem);
                Servico::verificaValorPrato(
                    $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
                    $sobrepor
                );
            }
        }
        tk_reload_data_with_filter('tab_ger04102_000_001', self::WINDOW);
        tk_reload_data_with_filter('tab_pla01500_000_003', 'ger04100');
        /* ID msg 1001331*/
        tk_message(
            tk_get_message_by_name('msg_ger04102_vrcustpadr_finalizado'),
            'success'
        );
    }


    /**
     * Verifica se as linhas modificadas estão marcadas
     * e confirma se há valores válidos para as linhas marcadas
     *
     * @param $linhasmarcadas
     * @param $porcentagem
     * @param $linhas
     *
     * @return bool|string
     */
    public static function linhasMarcadas(
        $linhasmarcadas, $porcentagem, $linhas
    )
    {
        if (empty($linhasmarcadas)) {
            return 'linhas';
        } else {
            if ($porcentagem) {
                return false;
            }
            $qtdmarc = count($linhasmarcadas);
            for ($contador = 0; $qtdmarc > $contador; $contador++) {
                $row = $linhasmarcadas[$contador];
                if ($row['NOVOVALOR'] == null) {
                    return 'valor';
                }
                $qtd = count($linhas);
                for ($i = 0; $qtd > $i; $i++) {
                    if ($i == $contador) {
                        $registro = $linhas[$i];
                        $row = $linhasmarcadas[$contador];
                        $diferenca = array_diff($row, $registro);
                        if (empty($diferenca) == false) {
                            return 'linhas';
                        }
                        $contador++;

                    } else {
                        $contador++;
                    }

                }

            }

        }
    }

    /**
     * Verifica se há linhas marcadas quando a atualização é feita por
     * porcentagem. E se há valor cadastrado para que haja o cálculo de
     * porcentagem.
     *
     * @param $linhasmarcadas
     *
     * @return bool|string
     */
    public static function percentual($linhasmarcadas)
    {
        $linhasOK = null;
        if (empty($linhasmarcadas)) {
            return 'linhas';
        } else {
            foreach ($linhasmarcadas as $registro) {
                if ($registro['VRCUSTPADR'] == null) {
                    $linhasOK = false;
                    return false;
                } else {
                    $linhasOK = true;
                }
            }
            if ($linhasOK) {
                return true;
            }
        }

    }

    /**
     * O novo valor digitado será ou inserido no intervalo de datas filtrado
     *
     * @param $registro
     * @param $p_dtini
     * @param $p_dtfin
     * @param $porcentagem
     * @param $idopcao
     * @param $sobrepor
     */
    public static function insereNovoValor(
        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao, $sobrepor
    )
    {

        if ($idopcao == 'I' AND $sobrepor == 'N') {
            $diainicio = DateUtil::getDataDeString($p_dtini);
            $diafinal = DateUtil::getDataDeString($p_dtfin);

            // $dtiniDia = $diainicio->format('d');

            $dtiniDia = DateUtil::getDia($diainicio);
            $dtiniMes = DateUtil::getMes($diainicio);
            $dtiniAno = DateUtil::getAno($diainicio);

            $dtfinMes = DateUtil::getMes($diafinal);

            /**
             * Percorre o intervalo dos meses e anos
             */
            $dtiniMes = (integer)$dtiniMes;
            $dtfinMes = (integer)$dtfinMes;

            $intervalo = 0;
            for ($inicial = $dtiniMes;$inicial != $dtfinMes;$inicial++) {
                if ($inicial == 13) {
                    $inicial = 1;
                    $intervalo++;
                }
                else {
                    $intervalo++;
                }
            }

            if ($intervalo<0) {
                $intervalo = $intervalo * -1;
            }
            while ($intervalo >= 0) {

                $dtiniMes = (integer)$dtiniMes;
                if ($dtiniMes == 13) {
                    $dtiniMes = 1;
                    $dtiniAno = $dtiniAno + 1;
                }
                if ($dtiniMes < 10) {
                    $mescorreto = '0';
                    $dtiniMes = (string)$dtiniMes;
                    $mescorreto .= $dtiniMes;

                    $datacorreta = array($dtiniDia, $mescorreto, $dtiniAno);
                    $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                    $dtiniMes = $dtiniMes + 1;
                } else {
                    $dtiniMes = (string)$dtiniMes;
                    $datacorreta = array($dtiniDia, $dtiniMes, $dtiniAno);
                    $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                    $dtiniMes = $dtiniMes + 1;
                }

                if ($registro['NOVOVALOR'] == null OR
                    $registro['NOVOVALOR'] == ''
                ) {
                    $registro['VRCUSTPADR'] = $porcentagem;
                } else {
                    $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
                }
                /**
                 * @var \Planejamento\Repositories\Custpadr $insere
                 */
                $insere = \Planejamento\Entities\Custpadr::getRepository();
                $insere->insereVrcustPadr(
                    $registro['CDFILIAL'],
                    $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                    $registro['VRCUSTPADR']
                );
                $intervalo--;
            }
        } else {
            if ($registro['NOVOVALOR'] == null OR $registro['NOVOVALOR'] == ''
            ) {
                $registro['VRCUSTPADR'] = $porcentagem;
            } else {
                $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
            }
            /**
             * @var \Planejamento\Repositories\Custpadr $insere
             */
            $insere = \Planejamento\Entities\Custpadr::getRepository();
            $insere->insereVrcustPadr(
                $registro['CDFILIAL'],
                $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                $registro['VRCUSTPADR']
            );
        }
    }

    /**
     * O Registro será alterado se o filtro for Incluir a alteração acontecerá
     * para todos registros do intervalo de datas, se Alterar ou Incluir e
     * sobrepor, a atualização aocntecerá na data específica.
     *
     * @param $registro
     * @param $p_dtini
     * @param $p_dtfin
     * @param $porcentagem
     * @param $idopcao
     * @param $sobrepor
     */
    public static function atualizaNovoValor(
        $registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao, $sobrepor
    )
    {
        $nrorg = Ambiente::getNrorg();

        if ($idopcao == 'I' AND $sobrepor == 'N') {
            $dtiniDia = DateUtil::getDia($p_dtini);
            $dtiniMes = DateUtil::getMes($p_dtini);
            $dtiniAno = DateUtil::getAno($p_dtini);

            $dtfinMes = DateUtil::getMes($p_dtfin);

            /**
             * Percorre o intervalo dos meses e anos
             */
            $dtiniMes = (integer)$dtiniMes;
            $dtfinMes = (integer)$dtfinMes;

            $intervalo = 0;
            for ($inicial = $dtiniMes;$inicial != $dtfinMes;$inicial++) {
                if ($inicial == 13) {
                    $inicial = 1;
                    $intervalo++;
                }
                else {
                    $intervalo++;
                }
            }

            if ($intervalo<0) {
                $intervalo = $intervalo * -1;
            }
            while ($intervalo >= 0) {

                $dtiniMes = (integer)$dtiniMes;
                if ($dtiniMes == 13) {
                    $dtiniMes = 1;
                    $dtiniAno = $dtiniAno + 1;
                }
                if ($dtiniMes < 10) {
                    $mescorreto = '0';
                    $dtiniMes = (string)$dtiniMes;
                    $mescorreto .= $dtiniMes;

                    $datacorreta = array($dtiniDia, $mescorreto, $dtiniAno);
                    $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                    $dtiniMes = $dtiniMes + 1;
                } else {
                    $dtiniMes = (string)$dtiniMes;
                    $datacorreta = array($dtiniDia, $dtiniMes, $dtiniAno);
                    $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                    $dtiniMes = $dtiniMes + 1;
                }

                if ($registro['NOVOVALOR'] == null OR
                    $registro['NOVOVALOR'] == ''
                ) {
                    $registro['VRCUSTPADR'] = $porcentagem;
                } else {
                    $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
                }

                /**
                 * @var \Planejamento\Repositories\Custpadr $atualiza Custpadr
                 */
                $existeCusto = Custpadr::getObjectsByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND DTREFCUSPADR = ? AND CDSERVFILI = ?',
                    array(
                        $registro['CDFILIAL'], $nrorg,
                        $registro['DTREFCUSPADR'], $registro['CDSERVFILI']
                    )
                );

                if ($existeCusto) {
                    $atualiza = Custpadr::getRepository();

                    $atualiza->atualizaVrcustPadr(
                        $registro['CDFILIAL'],
                        $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                        $registro['VRCUSTPADR']
                    );
                    $intervalo--;
                } else {
                    /**
                     * @var \Planejamento\Repositories\Custpadr $insere Custpadr
                     */
                    $insere = Custpadr::getRepository();
                    $insere->insereVrcustPadr(
                        $registro['CDFILIAL'],
                        $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                        $registro['VRCUSTPADR']
                    );
                    $intervalo--;
                }
            }
        } else {
            if ($registro['NOVOVALOR'] === null OR
                $registro['NOVOVALOR'] === ''
            ) {
                $registro['VRCUSTPADR'] = $porcentagem;
            } else {
                $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
            }

            /**
             * @var \Planejamento\Repositories\Custpadr $atualiza Custpadr
             */
            $atualiza = \Planejamento\Entities\Custpadr::getRepository();
            $atualiza->atualizaVrcustPadr(
                $registro['CDFILIAL'],
                $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                $registro['VRCUSTPADR']
            );
        }
    }

    /**
     * Atualiza o custo de acordo com a porcentagem dada no campo
     *
     * @param $linhasmarcadas
     * @param $porcentagem
     * @param $p_dtini
     * @param $p_dtfin
     * @param $idopcao
     * @param $sobrepor
     */
    public static function atualizaNovoValorPercentual(
        $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin, $idopcao, $sobrepor
    )
    {

        if ($idopcao == 'I' AND $sobrepor == 'N') {
            $i = 0;
            $nrorg = Ambiente::getNrorg();
            foreach ($linhasmarcadas as $registro) {


                if($registro['NOVOVALOR'] != ''){
                    self::atualizaNovoValor($registro, $p_dtini, $p_dtfin, $porcentagem, $idopcao, $sobrepor);
                    continue;
                }

                $dtiniDia = DateUtil::getDia($p_dtini);
                $dtiniMes = DateUtil::getMes($p_dtini);
                $dtiniAno = DateUtil::getAno($p_dtini);

                $dtfinMes = DateUtil::getMes($p_dtfin);

                /**
                 * Percorre o intervalo dos meses e anos
                 */
                $dtiniMes = (integer)$dtiniMes;
                $dtfinMes = (integer)$dtfinMes;
                $intervalo = 0;
                for ($inicial = $dtiniMes;$inicial != $dtfinMes;$inicial++) {
                    if ($inicial == 13) {
                        $inicial = 1;
                        $intervalo++;
                    }
                    else {
                        $intervalo++;
                    }
                }

                if ($intervalo<0) {
                    $intervalo = $intervalo * -1;
                }
                while ($intervalo >= 0) {

                    $dtiniMes = (integer)$dtiniMes;
                    if ($dtiniMes == 13) {
                        $dtiniMes = 1;
                        $dtiniAno = $dtiniAno + 1;
                    }
                    if ($dtiniMes < 10) {
                        $mescorreto = '0';
                        $dtiniMes = (string)$dtiniMes;
                        $mescorreto .= $dtiniMes;

                        $datacorreta = array($dtiniDia, $mescorreto, $dtiniAno);
                        $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                        $dtiniMes = $dtiniMes + 1;
                    } else {
                        $dtiniMes = (string)$dtiniMes;
                        $datacorreta = array($dtiniDia, $dtiniMes, $dtiniAno);
                        $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                        $dtiniMes = $dtiniMes + 1;
                    }
                    /**
                     * @var \Planejamento\Repositories\Custpadr $atualiza Custpadr
                     */
                    $existeCusto = Custpadr::getObjectsByWhere(
                        'CDFILIAL = ? AND NRORG = ? AND DTREFCUSPADR = ? AND CDSERVFILI = ?',
                        array(
                            $registro['CDFILIAL'], $nrorg,
                            $registro['DTREFCUSPADR'], $registro['CDSERVFILI']
                        )
                    );

                    if ($existeCusto) {
                        $atualiza = Custpadr::getRepository();
                        //$registro['NOVOVALOR'] = $registro['VRCUSTPADR'];
                        $registro['NOVOVALOR']
                            = $registro['VRCUSTPADR'] + ($registro['VRCUSTPADR']
                                * $porcentagem / 100);
                        $atualiza->atualizaVrcustPadr(
                            $registro['CDFILIAL'],
                            $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                            $registro['NOVOVALOR']
                        );
                        $intervalo--;
                    } else {
                        /**
                         * @var \Planejamento\Repositories\Custpadr $insere Custpadr
                         */
                        $insere = Custpadr::getRepository();
                        //$registro['NOVOVALOR'] = $registro['VRCUSTPADR'];
                        $registro['NOVOVALOR']
                            = $registro['VRCUSTPADR'] + ($registro['VRCUSTPADR']
                                * $porcentagem / 100);
                        $insere->insereVrcustPadr(
                            $registro['CDFILIAL'],
                            $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                            $registro['NOVOVALOR']
                        );
                    }
                    $intervalo--;
                }
                $linhasmarcadas[$i]['NOVOVALOR'] = $registro['NOVOVALOR'];
                $i++;


            }
        } else {
            $nrorg = Ambiente::getNrorg();
            foreach ($linhasmarcadas as $registro) {
                /**
                 * @var \Planejamento\Repositories\Custpadr $atualiza Custpadr
                 */
                $existeCusto = Custpadr::getObjectsByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND DTREFCUSPADR = ? AND CDSERVFILI = ?',
                    array(
                        $registro['CDFILIAL'], $nrorg,
                        $registro['DTREFCUSPADR'], $registro['CDSERVFILI']
                    )
                );

                if ($existeCusto) {
                    $atualiza = Custpadr::getRepository();
                    //$registro['NOVOVALOR'] = $registro['VRCUSTPADR'];
                    $registro['NOVOVALOR']
                        = $registro['VRCUSTPADR'] + ($registro['VRCUSTPADR']
                            * $porcentagem / 100);
                    $atualiza->atualizaVrcustPadr(
                        $registro['CDFILIAL'],
                        $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                        $registro['NOVOVALOR']
                    );
                } else {
                    /**
                     * @var \Planejamento\Repositories\Custpadr $insere Custpadr
                     */
                    $insere = Custpadr::getRepository();
                    //$registro['NOVOVALOR'] = $registro['VRCUSTPADR'];
                    $registro['NOVOVALOR']
                        = $registro['VRCUSTPADR'] + ($registro['VRCUSTPADR']
                            * $porcentagem / 100);
                    $insere->insereVrcustPadr(
                        $registro['CDFILIAL'],
                        $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
                        $registro['NOVOVALOR']
                    );
                }
            }
        }
        tk_reload_data_with_filter('tab_ger04102_000_001', self::WINDOW);
        tk_reload_data_with_filter('tab_pla01500_000_003', 'ger04100');
        Servico::verificaValorPrato(
            $linhasmarcadas, $porcentagem, $p_dtini, $p_dtfin, $idopcao,
            $sobrepor
        );
    }

    /**
     * Insere Custo quando o filtro vem com o parametro sobrepor marcado
     *
     * @param $registro
     * @param $porcentagem
     */
    public static function insereNovoValorSobrepor($registro, $porcentagem)
    {

        if ($registro['NOVOVALOR'] == null OR $registro['NOVOVALOR'] == '') {
            $registro['VRCUSTPADR'] = $porcentagem;
        } else {
            $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
        }
        /**
         * @var \Planejamento\Repositories\Custpadr $insere
         */
        $insere = \Planejamento\Entities\Custpadr::getRepository();
        $insere->insereVrcustPadr(
            $registro['CDFILIAL'],
            $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
            $registro['VRCUSTPADR']
        );

    }

    /**
     * Atualiza Custo quando o filtro vem com o parametro sobrepor marcado
     *
     * @param $registro
     * @param $porcentagem
     */
    public static function atualizaNovoValorSobrepor($registro, $porcentagem)
    {

        if ($registro['NOVOVALOR'] == null OR $registro['NOVOVALOR'] == '') {
            $registro['VRCUSTPADR'] = $porcentagem;
        } else {
            $registro['VRCUSTPADR'] = $registro['NOVOVALOR'];
        }

        /**
         * @var \Planejamento\Repositories\Custpadr $atualiza Custpadr
         */
        $atualiza = \Planejamento\Entities\Custpadr::getRepository();
        $atualiza->atualizaVrcustPadr(
            $registro['CDFILIAL'],
            $registro['CDSERVFILI'], $registro['DTREFCUSPADR'],
            $registro['VRCUSTPADR']
        );

    }

    /**
     * Custo Atual: 5
     *  1º Tipo de prato: 3
     *  2ª Tipo de prato: 2
     *  Manutenção automática
     *  Novo valor: 6
     *   Cálculo
     *   % = (1º Tipo de prato * 100 ) / Custo Atual
     *   Diferença entre os valores de custo = Custo Atual - Novo valor
     *   Total a ser somado ou substraído ao novo valor do tipo de prato =
     *   (Diferença entre os valores de custo * %) / 100
     *   1º Tipo de prato + Total
     *
     * @param $linhas
     * @param $porcentagem
     * @param $p_dtini
     * @param $p_dtfin
     * @param $idopcao
     * @param $sobrepor
     */
    public static function verificaValorPrato(
        $linhas, $porcentagem, $p_dtini, $p_dtfin, $idopcao, $sobrepor
    )
    {
        $connection = Registry::getEntityManager();
        $connection->beginTransaction();

        $iddestino = tk_get_env_var('IDDESTINO');
        if ($idopcao == 'A' OR $sobrepor == 'S') {
            foreach ($linhas as $registro) {
                $nrorg = Ambiente::getNrorg();
                if ($registro['NOVO_VALOR'] == null AND $iddestino == 'P') {
                    $registro['NOVOVALOR']
                        = $registro['VRCUSTPADR'] + ($registro['VRCUSTPADR']
                            * $porcentagem / 100);
                }
                if ($registro['NOVOVALOR'] == null AND $iddestino == 'V') {
                    $registro['NOVOVALOR'] = $porcentagem;
                }

                /**
                 * Linhas do Tipo de prato
                 *
                 * @var \Planejamento\Entities\Custpadrtp $dbRows Custpadrtp
                 */
                $dbRows = \Planejamento\Entities\Custpadrtp::getObjectsByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND DTREFCUSPADR = ? AND CDSERVFILI = ?',
                    array(
                        $registro['CDFILIAL'], $nrorg,
                        $registro['DTREFCUSPADR'], $registro['CDSERVFILI']
                    )
                );
                foreach ($dbRows as $dbrow) {
                    /**
                     * @var \Planejamento\Entities\Custpadrtp $dbrow Custpadrtp
                     */
                    $razao = ($dbrow->getVrcustpadrtp() * 100)
                        / $registro['VRCUSTPADR'];
                    $diferenca
                        = $registro['NOVOVALOR'] - $registro['VRCUSTPADR'];
                    $vrtipoprat = ($diferenca * $razao) / 100;

                    $novoTipoprat = $dbrow->getVrcustpadrtp() + ($vrtipoprat);
                    $dbrow->setVrcustpadrtp($novoTipoprat);

                    $dbrow->persist();
                }
            }
            tk_message(tk_get_message_by_name('msg_processo_sucesso'),'success');
        } else {
            foreach ($linhas as $registro) {

                $nrorg = Ambiente::getNrorg();
                $dtiniDia = DateUtil::getDia($p_dtini);
                $dtiniMes = DateUtil::getMes($p_dtini);
                $dtiniAno = DateUtil::getAno($p_dtini);

                $dtfinMes = DateUtil::getMes($p_dtfin);

                /**
                 * Percorre o intervalo dos meses e anos
                 */
                $dtiniMes = (integer)$dtiniMes;
                $dtfinMes = (integer)$dtfinMes;
                $intervalo = 0;
                for ($inicial = $dtiniMes;$inicial != $dtfinMes;$inicial++) {
                    if ($inicial == 13) {
                        $inicial = 1;
                        $intervalo++;
                    }
                    else {
                        $intervalo++;
                    }
                }

                if ($intervalo<0) {
                    $intervalo = $intervalo * -1;
                }
                while ($intervalo >= 0) {

                    $dtiniMes = (integer)$dtiniMes;
                    if ($dtiniMes == 13) {
                        $dtiniMes = 1;
                        $dtiniAno = $dtiniAno + 1;
                    }
                    if ($dtiniMes < 10) {
                        $mescorreto = '0';
                        $dtiniMes = (string)$dtiniMes;
                        $mescorreto .= $dtiniMes;

                        $datacorreta = array($dtiniDia, $mescorreto, $dtiniAno);
                        $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                        $dtiniMes = $dtiniMes + 1;
                    } else {
                        $dtiniMes = (string)$dtiniMes;
                        $datacorreta = array($dtiniDia, $dtiniMes, $dtiniAno);
                        $registro['DTREFCUSPADR'] = implode("/", $datacorreta);
                        $dtiniMes = $dtiniMes + 1;
                    }
                    if ($registro['NOVOVALOR'] == null) {
                        $registro['NOVOVALOR'] = $porcentagem;
                    }

                    /**
                     * Linhas do Tipo de prato
                     *
                     * @var \Planejamento\Entities\Custpadrtp $dbRows Custpadrtp
                     */
                    $dbRows
                        = \Planejamento\Entities\Custpadrtp::getObjectsByWhere(
                        'CDFILIAL = ? AND NRORG = ? AND DTREFCUSPADR = ? AND CDSERVFILI = ?',
                        array(
                            $registro['CDFILIAL'], $nrorg,
                            $registro['DTREFCUSPADR'], $registro['CDSERVFILI']
                        )
                    );
                    foreach ($dbRows as $dbrow) {
                        /**
                         * @var \Planejamento\Entities\Custpadrtp $dbrow Custpadrtp
                         */
                        $razao = ($dbrow->getVrcustpadrtp() * 100)
                            / $registro['VRCUSTPADR'];
                        $diferenca
                            = $registro['NOVOVALOR'] - $registro['VRCUSTPADR'];
                        $vrtipoprat = ($diferenca * $razao) / 100;

                        $novoTipoprat
                            = $dbrow->getVrcustpadrtp() + ($vrtipoprat);
                        $dbrow->setVrcustpadrtp($novoTipoprat);

                        $dbrow->persist();
                    }
                    $intervalo--;
                }
            }
        }
        $connection->flush();
        $connection->commit();
        tk_message(tk_get_message_by_name('msg_processo_sucesso'),'success');
    }

    /**
     * Exporta a configuração padrão
     *
     * @param $nrorg
     * @param $cdfiliorig
     * @param $cdservorig
     * @param $listafiliserv
     * @param $listafili
     * @param $idformulario
     * @param $idservico
     *
     * @throws \Exception
     * @return null
     */
    public static function exportarConfiguracaoPadrao(
        $nrorg, $cdfiliorig, $cdservorig, $listafiliserv,
        $listafili, $idformulario, $idservico
    )
    {
        try {
            /** @var $confcard \Planejamento\Repositories\Confcard */
            $confcard = \Planejamento\Entities\Confcard::getRepository();
            /** @var \Geral\Repositories\Servico $servico */
            $servico = \Geral\Entities\Servico::getRepository();
            /** @var \Geral\Entities\Servico $dadosFormulario */
            $dadosFormulario = \Geral\Entities\Servico::getDadosFormulario(
                $nrorg, $cdfiliorig, $cdservorig
            );


            if ($idformulario == self::SIM) {

                $hrinicaten = $dadosFormulario->getHrinicaten();
                $cdrotatrans = $dadosFormulario->getCdrotatrans();
                $hrfinaserv = $dadosFormulario->getHrfinaserv();
                $idservativ = $dadosFormulario->getIdservativ();
                $hrentrega = $dadosFormulario->getHrentrega();
                $cdserpaddie = $dadosFormulario->getCdserpaddie();
                $idtiposerv = $dadosFormulario->getIdtiposerv();
                $idconsservca = $dadosFormulario->getIdconsservca();
                $idbloqconfixper = $dadosFormulario->getIdbloqconfixper();
                $idutretiprat = $dadosFormulario->getIdutretiprat();
                $idbloqaltpra = $dadosFormulario->getIdbloqaltpra();
                $idconscomfat = $dadosFormulario->getIdconscomfat();
                $idservprinc = $dadosFormulario->getIdservprinc();
                $idservtran = $dadosFormulario->getIdservtran();
                $idvlrvardata = $dadosFormulario->getIdvlrvardata();
                $idservfatsep = $dadosFormulario->getIdservfatsep();
                $idcarddescentr = $dadosFormulario->getIdcarddescentr();

                $servico->atualizaParametrosServico(
                    $cdfiliorig, $cdservorig, $listafiliserv, $hrinicaten,
                    $cdrotatrans, $hrfinaserv, $idservativ, $hrentrega,
                    $cdserpaddie,
                    $idtiposerv, $idconsservca, $idbloqconfixper, $idutretiprat,
                    $idbloqaltpra, $idconscomfat, $idservprinc, $idservtran,
                    $idvlrvardata, $idservfatsep, $idcarddescentr
                );
            }
            if ($idservico == self::SIM) {
                $servico->incluirServicoExportacao(
                    $cdfiliorig, $cdservorig, $listafili
                );

                $listafiliserv = self::montaNovaListaFiliServ($listafiliserv, $cdservorig, $listafili);
            }

            $excluido = $confcard->excluiConfiguracaoPadrao(
                $cdfiliorig, $cdservorig, $listafiliserv
            );
            if ($excluido) {
                $exportado = $confcard->exportaConfiguracaoPadrao(
                    $cdfiliorig, $cdservorig, $listafiliserv
                );
            }
            tk_message(tk_get_message_by_name('msg_configuracao_padrao_exportada'), 'success');

        } catch (\Exception $e) {
            $mensagemexception = $e->getMessage();
            if(strpos($mensagemexception, "ORA-02292")){
                $restricaointegridade = tk_get_message_by_name('msg_ger04106_restricao_integridade');
                preg_match_all("/".$restricaointegridade." \\((.*)\\)/",$mensagemexception, $arrayerro);
                $erro = $arrayerro[1][0];
                $erro = explode('.', $erro);
                tk_message(Confcard::montaMensagemErroIntegridade($erro[0], $erro[1]), 'error');
            }else{
                throw $e;
            }

        }
    }


    /**
     * Monta nova lista baseada nos novos serviços que foram incluídos.
     * Pode retornar a lista completa (para usar na exportação) ou a lista com os serviços recém incluídos (para atuali
     * zar os dados do formulário).
     * */
    public static function montaNovaListaFiliServ($listafiliserv, $cdservorig, $listafilistring){
        $listafiliarray = explode(',', $listafilistring);
        $listafiliservnovos = '';

        foreach($listafiliarray as $fili){

            $listafiliservnovos = $listafiliservnovos.$fili.$cdservorig.'_';
        }

        return $listafiliserv.$listafiliservnovos;

    }



    /**
     * Com a mudança do volume proposto na Tela de clientes por serviço as quantidades na
     * tab Tabela de correção devem ser recalculados
     * @param $volume
     * @param $cdfilial
     * @param $cdservfili
     * @param $cdcliente
     *
     * @return null
     */
    public static function calcQtd($volume, $cdfilial, $cdservfili, $cdcliente)
    {
        try{
            /** @var  $tabela \Faturamento\Entities\Corrprecserv */
            $tabela = \Faturamento\Entities\Corrprecserv::getObjectsByWhere(
                'CDFILIAL = ? AND CDSERVFILI = ? AND CDCLIENTE = ?',
                array($cdfilial, $cdservfili, $cdcliente)
            );
            if($tabela){
                $connection = Registry::getEntityManager();
                $connection->beginTransaction();
                foreach($tabela as $linha){
                    /** @var  $linha \Faturamento\Entities\Corrprecserv */
                    $qtiniserv = ($volume - ($volume * $linha->getVrpeinicserv() / 100)) - 1;
                    //tk_set_field_value('QTINISERV', $qtiniserv, 'tab_fai00600_004');
                    $linha->setQtiniserv($qtiniserv);

                    $qtfinserv = ($volume - ($volume * $linha->getVrpefimserv() / 100)) - 1;
                    //tk_set_field_value('QTFINSERV', $qtfinserv, 'tab_fai00600_004');
                    $linha->setQtfinserv($qtfinserv);
                    $linha->persist();
                }
                $connection->flush();
                $connection->commit();
            }

        } catch(\Exception $e){

        }
    }

    public static function validaTabCorrecao($linhas, $cdfilial, $cdservfili, $cdcliente)
    {
        $tabela = \Faturamento\Entities\Corrprecserv::getObjectsByWhere(
            'CDFILIAL = ? AND CDSERVFILI = ? AND CDCLIENTE = ? ORDER BY VRPEINICSERV',
            array($cdfilial, $cdservfili, $cdcliente)
        );
        foreach($linhas as $linha){
            if (empty($linha['VRPEINICSERV'])) {
                tk_abort(tk_get_message_by_name('msg_fai00600_SemInicial'));
            }
            if (empty($linha['VRPEFIMSERV'])) {
                tk_abort(tk_get_message_by_name('msg_fai00600_SemFinal'));
            }
            if ($linha['VRPEINICSERV'] >= 100 OR $linha['VRPEINICSERV'] < 0) {
                tk_abort(
                    tk_get_message_by_name(
                        'msg_fai00600_vrpeiniserv_menorQueCem'
                    ), 'error'
                );
            }

            if (($linha['VRPEFIMSERV'] >= 100 OR $linha['VRPEFIMSERV'] < 0)) {
                tk_abort(
                    tk_get_message_by_name(
                        'msg_fai00600_vrpefimserv_menorQueCem'
                    ), 'error'
                );
                tk_cancel();

            }

            if ($linha['VRPEINICSERV'] >= $linha['VRPEFIMSERV']) {
                tk_abort(
                    tk_get_message_by_name(
                        'msg_fai00600_vrpefimservMaiorQuevrpeiniserv'
                    ), 'error'
                );
                tk_cancel();
            }

            if($linha['_STATE'] == 'U'){
                foreach($tabela as $registro){
                    /** @var  $registro \Faturamento\Entities\Corrprecserv */
                    if($linha['VRPEINICSERV_OLD'] == $registro->getVrpeinicserv()){
                        continue;
                    }
                    $inicialAlt = $linha['VRPEINICSERV'];
                    $finalAlt = $linha['VRPEFIMSERV'];
                    $inicio = $registro->getVrpeinicserv();
                    $final = $registro->getVrpefimserv();

                    $inicialAlt = $inicialAlt*100;
                    $finalAlt = $finalAlt*100;

                    $inicio = $inicio*100;
                    $final = $final*100;

                    for($i = $inicio;$i <= $final;$i++){
                        for($j = $inicialAlt;$j <= $finalAlt;$j++){
                            if($i==$j){
                                return 'inicio';
                            }
                        }
                    }

                }
            }
            else{
                foreach($tabela as $registro){
                    /** @var  $registro \Faturamento\Entities\Corrprecserv */
                    $inicialAlt = $linha['VRPEINICSERV'];
                    $finalAlt = $linha['VRPEFIMSERV'];
                    $inicio = $registro->getVrpeinicserv();
                    $final = $registro->getVrpefimserv();

                    $inicialAlt = $inicialAlt*100;
                    $finalAlt = $finalAlt*100;

                    $inicio = $inicio*100;
                    $final = $final*100;

                    for($i = $inicio;$i <= $final;$i++){
                        for($j = $inicialAlt;$j <= $finalAlt;$j++){
                            if($i==$j){
                                return 'inicio';
                            }
                        }
                    }
                }
            }

            return false;
        }


    }
}